
// TestSocket.cpp : Defines the class behaviors for the application.
//

#include "stdafx.h"
#include "VaxPipeThread.h"
#include "VaxPipeClient.h"

#define VAX_SYSTEM_THREAD_MSG_QUIT	101

////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////

CVaxPipeThread::CVaxPipeThread()
{
	m_hThread = NULL;
}

CVaxPipeThread::~CVaxPipeThread()
{
	StopThread();
}

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

typedef struct SYS_THREAD_PROC_PARAM
{
	CVaxPipeThread* m_pPipeThread;
	LPCTSTR m_sPipeName[2];
	HANDLE m_hWaitCreated;

} SysThreadProcParam;

////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

BOOLEAN CVaxPipeThread::StartThread(LPCTSTR sPipeNameZERO, LPCTSTR sPipeNameONE)
{
	HANDLE hWaitCreated = ::CreateEvent(NULL, FALSE, FALSE, NULL);
	if (hWaitCreated == NULL) return FALSE;

	/////////////////////////////////////////////////////////////////////

	SysThreadProcParam vThreadTLS;

	vThreadTLS.m_pPipeThread = this;
	vThreadTLS.m_sPipeName[0] = sPipeNameZERO;
	vThreadTLS.m_sPipeName[1] = sPipeNameONE;
	vThreadTLS.m_hWaitCreated = hWaitCreated;
	
	/////////////////////////////////////////////////////////////////////
		
	m_hThread = ::CreateThread(NULL, 0, CVaxPipeThread::VaxSysThreadProcUI, &vThreadTLS, 0, &m_dwThreadId);

	if (m_hThread == NULL)
	{
		::CloseHandle(hWaitCreated);
		return FALSE;
	}

	::WaitForSingleObject(hWaitCreated, INFINITE);
	::CloseHandle(hWaitCreated);

	return TRUE;
}

void CVaxPipeThread::StopThread()
{
	if (m_hThread == NULL)
		return;

	::PostThreadMessage(m_dwThreadId, VAX_SYSTEM_THREAD_MSG_QUIT, NULL, NULL);

	::WaitForSingleObject(m_hThread, INFINITE);
	::CloseHandle(m_hThread);

	m_hThread = NULL;
}

////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////

DWORD WINAPI CVaxPipeThread::VaxSysThreadProcUI(LPVOID pParam)
{
	SysThreadProcParam* pProcParam = (SysThreadProcParam*) pParam;
	
	CString sPipeName[2];

	sPipeName[0] = pProcParam->m_sPipeName[0];
	sPipeName[1] = pProcParam->m_sPipeName[1];
	
	CVaxPipeThread* pPipeThread = pProcParam->m_pPipeThread;

	MSG hMsg;
	::PeekMessage(&hMsg, NULL, WM_USER, WM_USER, PM_NOREMOVE);  // Force the system to create the message queue.  REF: MSDN PostThreadMessage() details.
		
	::SetEvent(pProcParam->m_hWaitCreated);

	CVaxPipeClient objPipeClient[2];
		
	if(!objPipeClient[0].CreatePipe(sPipeName[0]))
		return 0;
	
	if (!objPipeClient[1].CreatePipe(sPipeName[1]))
		return 0;
	
	HANDLE aWaitEvent[2];
	DWORD dwWaitCount = 2;

	objPipeClient[0].BeginReadPipe(aWaitEvent[0]);
	objPipeClient[1].BeginReadPipe(aWaitEvent[1]);
				
	BOOLEAN bQuitThread = FALSE;
				
	while (bQuitThread == FALSE)
	{
		DWORD dwWaitIndex = ::MsgWaitForMultipleObjects(dwWaitCount, aWaitEvent, FALSE, INFINITE, QS_POSTMESSAGE);

		if ((dwWaitIndex == WAIT_FAILED) || (dwWaitIndex == WAIT_TIMEOUT))
		{
			bQuitThread = TRUE;
			break;
		}

		if ((WAIT_OBJECT_0 + dwWaitIndex) < dwWaitCount)
		{
			if(dwWaitIndex == 0)
				CVaxPipeThread::ProcessThreadWaitEvent(0, &objPipeClient[0], pPipeThread);

			else if (dwWaitIndex == 1)
				CVaxPipeThread::ProcessThreadWaitEvent(1, &objPipeClient[1], pPipeThread);

		}

		else if ((WAIT_OBJECT_0 + dwWaitIndex) >= dwWaitCount)
		{
			CVaxPipeThread::ProcessThreadPostedMsg(bQuitThread);
		}

	}

	objPipeClient[0].ClosePipe();
	objPipeClient[1].ClosePipe();

	return 0;
}

////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////

void CVaxPipeThread::ProcessThreadWaitEvent(int nMediaPos, CVaxPipeClient* pPipeClient, CVaxPipeThread* pPipeThread)
{
	char aDataReadPipe[64000];
	int nSizeReadPipe = 0;

	if (!pPipeClient->EndReadPipe(aDataReadPipe, nSizeReadPipe))
		return;
	
	HANDLE hResultWaitEvent = NULL;
	pPipeClient->BeginReadPipe(hResultWaitEvent);

	pPipeThread->OnVaxPipeClientRecvPCM(nMediaPos, aDataReadPipe, nSizeReadPipe);
}

BOOLEAN CVaxPipeThread::ProcessThreadPostedMsg(BOOLEAN& bQuitThread)
{
	MSG hMsg;
	::memset(&hMsg, 0, sizeof(MSG));

	if (!::PeekMessage(&hMsg, NULL, 0, 0, PM_REMOVE))
	{
		return FALSE;
	}

	if ((hMsg.message == VAX_SYSTEM_THREAD_MSG_QUIT) && (hMsg.hwnd == NULL))
	{
		bQuitThread = TRUE;
	}

	return TRUE;
}
