#pragma once
#include "afxwin.h"

///////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////

#define SIP_DEFAULT_PORT								6080

#define VAX_PEER_TYPE_USER								0
#define VAX_PEER_TYPE_LINE								1

#define VAX_LINE_TYPE_UDP								0
#define VAX_LINE_TYPE_TCP								1
#define VAX_LINE_TYPE_TLS								2

///////////////////////////////////////////////////////////

#define VAX_WAVE_REC_TYPE_MONO_PCM						0
#define VAX_WAVE_REC_TYPE_STEREO_PCM					1
#define VAX_WAVE_REC_TYPE_MONO_G711U					2
#define VAX_WAVE_REC_TYPE_STEREO_G711U					3
#define VAX_WAVE_REC_TYPE_MONO_G711A					4
#define VAX_WAVE_REC_TYPE_STEREO_G711A					5

///////////////////////////////////////////////////////////

#define VAX_CODEC_GSM610								0
#define VAX_CODEC_ILBC									1
#define VAX_CODEC_G711A									2
#define VAX_CODEC_G711U									3
#define VAX_CODEC_G729									4

#define VAX_CODEC_TOTAL_COUNT							5

///////////////////////////////////////////////////////////

#define VAX_CUSTOM_HEADER_REQ_ID_INVITE					0

///////////////////////////////////////////////////////////

#define VAX_CLOSED_REASON_CODE_HANGUP					0
#define VAX_CLOSED_REASON_CODE_SESSION_LOST				1
#define VAX_CLOSED_REASON_CODE_MOVED					2
#define VAX_CLOSED_REASON_CODE_TRANSFERED				3
#define VAX_CLOSED_REASON_CODE_REJECTED					4
#define VAX_CLOSED_REASON_CODE_FAILED					5
#define VAX_CLOSED_REASON_CODE_CANCELLED				6
#define VAX_CLOSED_REASON_CODE_TIMEOUT					7
#define VAX_CLOSED_REASON_CODE_CLOSED				    8

/////////////////////////////////////////////////////////

#define VAX_ACCESS_PCM_TYPE_ALL_NONE			-1
#define VAX_ACCESS_PCM_TYPE_EVENT_NONE			0
#define VAX_ACCESS_PCM_TYPE_EVENT_MEDIA_ZERO	1
#define VAX_ACCESS_PCM_TYPE_EVENT_MEDIA_ONE		2
#define VAX_ACCESS_PCM_TYPE_PIPE_NONE			3
#define VAX_ACCESS_PCM_TYPE_PIPE_MEDIA_ZERO		4
#define VAX_ACCESS_PCM_TYPE_PIPE_MEDIA_ONE		5


#define VAX_MEDIA_DETECT_VAD_TYPE_INBOUND				0
#define VAX_MEDIA_DETECT_VAD_TYPE_OUTBOUND				1

/////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////

#if WIN64
#import "..\..\Component (COM)\VaxTeleServerREC.dll" rename_namespace("VaxTeleServerREC")
#else

#import "..\Component (COM)\VaxTeleServerREC.dll" rename_namespace("VaxTeleServerREC")
#endif

using namespace VaxTeleServerREC;

class CVaxServerCOM : public CCmdTarget
{
	DECLARE_DYNCREATE(CVaxServerCOM)

public:
	CVaxServerCOM();
	~CVaxServerCOM();

public:

	DWORD m_dwCookie;
	
	_IVaxServerRECEvents* m_pVaxEventsREC;
	IVaxServerREC* m_pVaxREC;

	//------------------------------------------------------------------------------------------------------------------
	//-------------------------------			Functions Added
	//------------------------------------------------------------------------------------------------------------------
	
protected:

	BOOL LoadCOM();
	void UnLoadCOM();
	BOOL IsLoadedCOM();

public:
	
	LONG GetVaxErrorCode();
	CString GetVaxErrorText();

	void SetLicenseKey(LPCTSTR sLicenseKey);
	
	BOOL Initialize(LPCTSTR sDomainRealm);
	void UnInitialize();
		
	BOOL SetListenPortRangeRTP(LONG nListenStartPort, LONG nListenEndPort);
	BOOL SetNetworkMediaRTP(LPCTSTR sListenIP, LONG nListenStartPort);

	BOOL OpenNetworkUDP(LPCTSTR sListenIP, LONG nListenPort);
	BOOL OpenNetworkTCP(LPCTSTR sListenIP, LONG nListenPort);
	BOOL OpenNetworkTLS(LPCTSTR sListenIP, LONG nListenPort, LPCTSTR sCertPEM);

	void CloseNetworkUDP();
	void CloseNetworkTCP();
	void CloseNetworkTLS();

	BOOL CryptoMediaNONE(BOOL bForced);
	BOOL CryptoMediaSDP(BOOL bForced);
				
	BOOL AddUser(LPCTSTR sUserName, LPCTSTR sPassword, LPCTSTR sAudioCodecList);
	void RemoveUser(LPCTSTR sUserName);
	BOOL RegisterUserExpiry(LONG nExpiry);
	
	BOOL AttachRegister(LONG nRegId, LPCTSTR sUserName);
	BOOL AcceptRegister(LONG nRegId);
	BOOL RejectRegister(LONG nRegId, LONG nStatusCode, LPCTSTR sReasonPhrase);
	BOOL AuthRegister(LONG nRegId);

	BOOL AddLine(LPCTSTR sLineName, LONG nLineType, LPCTSTR sDisplayName, LPCTSTR sUserName, LPCTSTR sAuthLogin, LPCTSTR sAuthPwd, LPCTSTR sDomainRealm, LPCTSTR sServerAddr, LONG nServerPort, LPCTSTR sAudioCodecList);
	void RemoveLine(LPCTSTR sLineName);

	BOOL RegisterLine(LPCTSTR sLineName, LONG nExpire);
	BOOL UnRegisterLine(LPCTSTR sLineName);
		
	BOOL AcceptCallSession(LONG nSessionId, LONG nTimeout);
	BOOL RejectCallSession(LONG nSessionId, LONG nStatusCode, LPCTSTR sReasonPhrase);
	BOOL CloseCallSession(LONG nSessionId);
					
	//////////////////////////////////////////////////////////////////////////////

	BOOL RecordWaveSetCacheSize(LONG nCacheSize);
	BOOL RecordWaveStartToCallSession(LONG nSessionId, LPCTSTR sFileName, LONG nTypeREC);
	void RecordWaveStopToCallSession(LONG nSessionId);
	BOOL RecordWavePauseToCallSession(LONG nSessionId, BOOL bEnable);
			
	BOOL AudioSessionLost(BOOL bEnable, LONG nTimeout);  // Seconds.
	
	BOOL VaxServerStartTick(LONG nTickId, LONG nElapse); // Milli-Seconds.
	BOOL VaxServerStopTick(LONG nTickId);

	BOOL SetUserAgentName(LPCTSTR sName);
	CString GetUserAgentName();
	
	BOOL SetSessionNameSDP(LPCTSTR sName);
	CString GetSessionNameSDP();
		
	BOOL DiagnosticLogSIP(BOOL bInbound, BOOL bOutbound);
	
	BOOL CallSessionSendStatusResponse(LONG nSessionId, LONG nStatusCode, LPCTSTR sReasonPhrase, LPCTSTR sContact);

	BOOL AddCustomHeader(LONG nReqId, LPCTSTR sName, LPCTSTR sValue);
	BOOL RemoveCustomHeader(LONG nReqId, LPCTSTR sName);
	BOOL RemoveCustomHeaderAll(LONG nReqId);

	LONG GetCallSessionTxCodec(LONG nSessionId);
	LONG GetCallSessionRxCodec(LONG nSessionId);

public:

	LONG GetCallCount();

public:

	CString GetCallSessionHeaderCallId(LONG nSessionId);
	
	BOOL CallSessionMuteVoice(LONG nSessionId, BOOL bListen, BOOL bSpeak);
	CString GetUserAgentNameCall(LONG nSessionId);

	LONG GetCallSessionPeerType(LONG nSessionId);
	CString GetCallSessionPeerName(LONG nSessionId);
		
	///////////////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////////

	BOOL AddNetworkRouteSIP(LPCTSTR sAssignedIP, LPCTSTR sRouterIP);
	BOOL AddNetworkRouteRTP(LPCTSTR sAssignedIP, LPCTSTR sRouterIP);
				
	///////////////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////////

	BOOL AccessAudioPCM(LONG nSessionId, LPCTSTR sPipeName, LONG nAccessType);
		
	BOOL AttackDetectScanSIP(LPCTSTR sDomainName);
	BOOL AttackDetectFloodSIP(LONG nReqRecvLimit);
	BOOL AttackDetectBruteForceSIP(LONG nFailureCount, LONG nFailureInterval);
			
	///////////////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////////
	
	BOOL DetectStartVAD(LONG nSessionId, LONG nAudioLevelVU, LONG nSilenceLengthPCM, LONG nSilenceCountPCM, LONG nAnalysisLengthPCM, LONG nExpiryLengthPCM, LONG nProcessLengthPCM, LONG nDetectType);
	void DetectStopVAD(LONG nSessionId);

public:

	afx_msg virtual void OnFinalRelease();


	///////////////////////////////////////////////////////////////////////////////////////
	///////////////////////////////////////////////////////////////////////////////////////

protected:

	virtual void OnRegisterUser(LPCTSTR sUserName, LPCTSTR sDomain, LPCTSTR sUserAgent, LPCTSTR sFromIP, LONG nFromPort, LONG nRegId);
	virtual void OnRegisterUserSuccess(LPCTSTR sUserName, LPCTSTR sFromIP, LONG nFromPort, LONG nRegId);
	virtual void OnRegisterUserFailed(LPCTSTR sUserName, LPCTSTR sFromIP, LONG nFromPort, LONG nRegId);
	virtual void OnUnRegisterUser(LPCTSTR sUserName);
		
	virtual void OnIncomingCall(LONG nSessionId, LPCTSTR sCallerName, LPCTSTR sCallerId, LPCTSTR sCalleeName, LPCTSTR sCalleeId, LPCTSTR sDialNo, LONG nFromPeerType, LPCTSTR sFromPeerName, LPCTSTR sUserAgent, LPCTSTR sRecXML, LPCTSTR sFromIP, UINT nFromPort);
	virtual void OnCallSessionConnected(LONG nSessionId);

	virtual void OnCallSessionCreated(LONG nSessionId, LONG nReasonCode);
	virtual void OnCallSessionClosed(LONG nSessionId, LONG nReasonCode);

	virtual void OnCallSessionLost(LONG nSessionId);
	virtual void OnCallSessionHangup(LONG nSessionId);
	virtual void OnCallSessionTimeout(LONG nSessionId);
	virtual void OnCallSessionCancelled(LONG nSessionId);

	virtual void OnLineRegisterTrying(LPCTSTR sLineName);
	virtual void OnLineRegisterFailed(LPCTSTR sLineName, LONG nStatusCode, LPCTSTR sReasonPhrase);
	virtual void OnLineRegisterSuccess(LPCTSTR sLineName);

	virtual void OnLineUnRegisterTrying(LPCTSTR sLineName);
	virtual void OnLineUnRegisterFailed(LPCTSTR sLineName, LONG nStatusCode, LPCTSTR sReasonPhrase);
	virtual void OnLineUnRegisterSuccess(LPCTSTR sLineName);

	virtual void OnOutgoingDiagnosticLog(LPCTSTR sMsgSIP, LPCTSTR sToIP, LONG nToPort);
	virtual void OnIncomingDiagnosticLog(LPCTSTR sMsgSIP, LPCTSTR sFromIP, LONG nFromPort);
		
	virtual void OnVaxServerTick(LONG nTickId);

	virtual void OnCallSessionRecordedWaveREC(LONG nSessionId, char* pDataREC, LONG nSizeREC, LONG nTypeREC);

	virtual void OnCallSessionErrorLog(LONG nSessionId, LONG nErrorCode, LPCTSTR sErrorMsg);
	virtual void OnVaxErrorLog(LPCTSTR sFuncName, LONG nErrorCode, LPCTSTR sErrorMsg);
	
	virtual void OnVaxLicenseChannel(LONG nTotalChannels, LONG nUsedChannels);
	virtual void OnCallSessionAccessAudioPCM(LONG nSessionId, LONG nAccessType, char* pDataPCM, LONG nSizePCM, LONG nTypePCM);

	virtual void OnAttackDetectedScanSIP(LPCTSTR sReqMethod, LPCTSTR sAddrIP, LONG nAddrPort, LONG nAddrType);
	virtual void OnAttackDetectedFloodSIP(LPCTSTR sAddrIP, LONG nAddrPort, LONG nAddrType);
	virtual void OnAttackDetectedBruteForceSIP(LPCTSTR sReqMethod, LONG nAuthFailureCount, LPCTSTR sAddrIP, LONG nAddrPort, LONG nAddrType);

	virtual void OnCallSessionMediaDetectStartedVAD(LONG nSessionId);
	virtual void OnCallSessionMediaDetectCompletedVAD(LONG nSessionId, char* pDataPCM, LONG nSizePCM, LONG nTypePCM);
	virtual void OnCallSessionMediaDetectExpiredVAD(LONG nSessionId);

	DECLARE_MESSAGE_MAP()
	// Generated OLE dispatch map functions
	//{{AFX_DISPATCH(CVaxServerCOM)
						
	//}}AFX_DISPATCH

	DECLARE_DISPATCH_MAP()
	DECLARE_INTERFACE_MAP()

};

